//
// "$Id$"
//
// Fl_Rect test program for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2017 by Bill Spitzak and others.
//
// This library is free software. Distribution and use rights are outlined in
// the file "COPYING" which should have been included with this file.  If this
// file is missing or damaged, see the license at:
//
//     http://www.fltk.org/COPYING.php
//
// Please report all bugs and problems on the following page:
//
//     http://www.fltk.org/str.php
//

// Build and test:
//
// (1) save Fl_Rect.H as FL/Fl_Rect.H
// (2) `fltk-config --compile rect.cxx'
// (3) ./rect

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Button.H>

#include <FL/Fl_Rect.H>

#include <stdio.h>

// MyWindow class -- new constructors and public sizes_rect2() methods

class MyWindow : public Fl_Window {

public:

  MyWindow(int X, int Y, int W, int H, const char *L)
  : Fl_Window(X,Y,W,H,L) {}

  MyWindow(int W, int H)
  : Fl_Window(W,H) {}

  MyWindow(Fl_Rect R, const char *L = 0)
  : Fl_Window(R.x(), R.y(), R.w(), R.h(), L) {}

  int *sizes() { return Fl_Window::sizes(); }	// make it public (!)

  Fl_Rect2 *sizes_rect2() {
    return (Fl_Rect2*)Fl_Window::sizes();
  }

  Fl_Rect2 *sizes_rect2(int i) {
    Fl_Rect2 *s = (Fl_Rect2*)Fl_Window::sizes();
    return &s[i];
  }

}; // class MyWindow


// MyButton class with new constructor using Fl_Rect

class MyButton : public Fl_Button {

public:

  MyButton(int X, int Y, int W, int H, const char *L = 0)
  : Fl_Button(X,Y,W,H,L) {
    color(FL_YELLOW);
  }

  MyButton(Fl_Rect R, const char *L = 0)
  : Fl_Button(R.x(), R.y(), R.w(), R.h(), L) {
    color(FL_GREEN);
  }

}; // class MyButton

// Globals

Fl_Rect		rwin (  0,   0, 340, 280 );	// window position and size
Fl_Rect		rbx  ( 20,  40, 300, 100 );	// box position and size

// Fl_Rect	rbt  ( 40, 160, 260,  80 );	     // FLTK standard (X,Y,W,H)
Fl_Rect		rbt  ( Fl_Rect2(40, 300, 160, 240) ); // same as above (L,R,T,B)

Fl_Rect		sizes[10];	// test: sizeof()
Fl_Rect2	sizes2[10];	// test: sizeof()

// Callback to show (printf()) the (re-initialized) sizes() array

void show_cb(Fl_Widget *b, void *) {

  MyWindow *w = (MyWindow *)b->window();
  const int nc = w->children();
  w->init_sizes();		// reset sizes()

  // print sizes_rect() with conversion to Fl_Rect

  Fl_Rect2 *rp = w->sizes_rect2();	// get sizes() as Fl_Rect2*

  for (int i=0; i<nc+2; i++, rp++) {
    // Fl_Rect r = *rp;			// assignment with conversion
    Fl_Rect r = w->sizes_rect2(i);	// uses Fl_Rect's c'tor (Fl_Rect2 *)
    printf("[%2d] x=%3d, y=%3d, w=%3d, h=%3d, r=%3d, b=%3d : ",
	   i-2, r.x(), r.y(), r.w(), r.h(), r.right(), r.bottom());

    // print widget label
    if (i==0) printf("<window>");
    else if (i==1) printf("<resizable>");
    else printf("'%s'", w->child(i-2)->label()?w->child(i-2)->label():"<no label>");
    printf("\n");

    // print Fl_Rect (commented out)
    // r.print("xxx");
  }

  // print again: now original int * array sizes():

  int *s = w->sizes();		// get sizes() as int*

  for (int i=0; i<nc+2; i++, s += 4) {
    printf("[%2d] left=%3d, right=%3d, top=%3d, bottom=%3d\n",
	   i-2, s[0], s[1], s[2], s[3]);
  }

  printf("\n");			// add empty line
  fflush(stdout);		// flush output buffer

} // show_cb()


// main test program

int main(int argc, char **argv) {

  printf("sizeof(Fl_Rect ) = %d\n", (int)sizeof(Fl_Rect));
  printf("sizeof(Fl_Rect2) = %d\n", (int)sizeof(Fl_Rect2));
  printf("sizeof(rwin     ) = %d\n", (int)sizeof(rwin));
  printf("sizeof(sizes   ) = %d\n", (int)sizeof(sizes));
  printf("sizeof(sizes2  ) = %d\n", (int)sizeof(sizes2));

  Fl_Rect rb2 = rbt;
  rbt.print("rbt");
  rb2.print("rb2");

//MyWindow *window = new MyWindow(rwin.w(),rwin.h());
  MyWindow *window = new MyWindow(rwin);

  Fl_Box *box = new Fl_Box(rbx.x(), rbx.y(), rbx.w(), rbx.h(), "Hello, World!");
  box->box(FL_UP_BOX);
  box->labelfont(FL_BOLD+FL_ITALIC);
  box->labelsize(36);
  box->labeltype(FL_SHADOW_LABEL);

  const char *bt_label = "Resize window and click\n"
			 "this button to display (print)\n"
			 "sizes() in the console (stdout)";

#if (0)		// standard constructor

  MyButton *button = new MyButton(rbt.x(), rbt.y(), rbt.w(), rbt.h(), bt_label);

#elif (0)	// new constructor with Fl_Rect variable

  MyButton *button = new MyButton(rbt, bt_label);

#elif (0)	// new constructor with explicit Fl_Rect (temporary)

  MyButton *button = new MyButton(Fl_Rect(40,160,260,80), bt_label);
  button->color(FL_WHITE);

#else		// new constructor with Fl_Rect2 (implicitly converted to Fl_Rect)

  MyButton *button = new MyButton(Fl_Rect2(40,300,160,240), bt_label);
  button->color(fl_lighter(FL_RED));

#endif		// different constructors

  button->callback(show_cb);
  button->align(FL_ALIGN_WRAP);

  window->end();
  window->resizable(box);
  window->size_range(rwin.w(),rwin.h());
  window->show(argc, argv);

  return Fl::run();
}

//
// End of "$Id$".
//
